@@ -2,10 +2,13 @@
 Contains code to generate pydantic v2 models from json schemas.
 Since the used tool doesn't support all features we need, we monkey patch some functions.
 """
+
 import itertools
 import re
+import shutil
+from enum import Enum
 from pathlib import Path
-from typing import Sequence, Tuple, Type
+from typing import Any, Sequence, Type
 
 import datamodel_code_generator.parser.base
 import datamodel_code_generator.reference
@@ -17,6 +20,17 @@ from datamodel_code_generator.parser.jsonschema import JsonSchemaParser
 from datamodel_code_generator.types import DataType, StrictTypes, Types
 
 from bo4e_generator.schema import SchemaMetadata
+from bo4e_generator.sqlparser import adapt_parse_for_sql, remove_pydantic_field_import, write_many_many_links
+
+
+class OutputType(str, Enum):
+    """
+    enum to specify the output type
+    """
+
+    PYDANTIC_V2 = "pydantic_v2"
+    PYDANTIC_V1 = "pydantic_v1"
+    SQL_MODEL = "sql_model"
 
 
 def get_bo4e_data_model_types(
@@ -37,7 +51,7 @@ def get_bo4e_data_model_types(
     def _module_path(self) -> list[str]:
         if self.name not in namespace:
             raise ValueError(f"Model not in namespace: {self.name}")
-        return [namespace[self.name].pkg, namespace[self.name].module_name]
+        return list(namespace[self.name].module_path)
 
     @property  # type: ignore[misc]
     # "property" used with a non-method
@@ -93,7 +107,7 @@ def monkey_patch_relative_import():
     Anyway, this monkey patch changes the imports to "from ..enum.typ import Typ" which resolves the issue.
     """
 
-    def relative(current_module: str, reference: str) -> Tuple[str, str]:
+    def relative(current_module: str, reference: str) -> tuple[str, str]:
         """Find relative module path."""
 
         current_module_path = current_module.split(".") if current_module else []
@@ -155,9 +169,7 @@ def bo4e_init_file_content(namespace: dict[str, SchemaMetadata], version: str) -
     init_file_content += "]\n\n"
 
     for schema_metadata in namespace.values():
-        init_file_content += (
-            f"from .{schema_metadata.pkg}.{schema_metadata.module_name} import {schema_metadata.class_name}\n"
-        )
+        init_file_content += f"from .{'.'.join(schema_metadata.module_path)} import {schema_metadata.class_name}\n"
     init_file_content += "\nfrom .__version__ import __version__\n"
 
     return init_file_content
@@ -171,18 +183,29 @@ def remove_future_import(python_code: str) -> str:
 
 
 def parse_bo4e_schemas(
-    input_directory: Path, namespace: dict[str, SchemaMetadata], pydantic_v1: bool = False
+    input_directory: Path, namespace: dict[str, SchemaMetadata], output_type: OutputType
 ) -> dict[Path, str]:
     """
     Generate all BO4E schemas from the given input directory. Returns all file contents as dictionary:
     file path (relative to arbitrary output directory) => file content.
     """
     data_model_types = get_bo4e_data_model_types(
-        DataModelType.PydanticBaseModel if pydantic_v1 else DataModelType.PydanticV2BaseModel,
+        (
+            DataModelType.PydanticBaseModel
+            if output_type is OutputType.PYDANTIC_V1.name
+            else DataModelType.PydanticV2BaseModel
+        ),
         target_python_version=PythonVersion.PY_311,
         namespace=namespace,
     )
     monkey_patch_relative_import()
+
+    additional_arguments: dict[str, Any] = {}
+
+    if output_type is OutputType.SQL_MODEL.name:
+        # adapt input for SQLModel classes
+        namespace, additional_arguments, input_directory, links = adapt_parse_for_sql(input_directory, namespace)
+
     parser = JsonSchemaParser(
         input_directory,
         data_model_type=data_model_types.data_model,
@@ -190,12 +213,13 @@ def parse_bo4e_schemas(
         data_model_field_type=data_model_types.field_model,
         data_type_manager_type=data_model_types.data_type_manager,
         dump_resolve_reference_action=data_model_types.dump_resolve_reference_action,
-        # use_annotated=not pydantic_v1,
+        # use_annotated=OutputType is not OutputType.PYDANTIC_V1.name,
         use_double_quotes=True,
         use_schema_description=True,
         use_subclass_enum=True,
         use_standard_collections=True,
         use_union_operator=True,
+        use_field_description=True,
         set_default_enum_member=True,
         snake_case_field=True,
         field_constraints=True,
@@ -205,17 +229,17 @@ def parse_bo4e_schemas(
         allow_extra_fields=False,
         allow_population_by_field_name=True,
         use_default_kwarg=True,
+        **additional_arguments,
     )
     parse_result = parser.parse()
     if not isinstance(parse_result, dict):
         raise ValueError(f"Unexpected type of parse result: {type(parse_result)}")
     file_contents = {}
     for schema_metadata in namespace.values():
+        module_path = schema_metadata.module_path_with_extension
         if schema_metadata.module_name.startswith("_"):
             # Because somehow the generator uses the prefix also on the module name. Don't know why.
-            module_path = (schema_metadata.pkg, f"field{schema_metadata.module_name}.py")
-        else:
-            module_path = (schema_metadata.pkg, f"{schema_metadata.module_name}.py")
+            module_path = *module_path[:-1], f"field{module_path[-1]}"
 
         if module_path not in parse_result:
             raise KeyError(
@@ -225,8 +249,19 @@ def parse_bo4e_schemas(
                 # Somehow, mypy is not good enough to understand the instance-check above
             )
 
-        file_contents[schema_metadata.output_file] = remove_future_import(parse_result.pop(module_path).body)
+        python_code = remove_future_import(parse_result.pop(module_path).body)
+        if output_type is OutputType.SQL_MODEL.name:
+            # remove pydantic field
+            python_code = remove_pydantic_field_import(python_code)
+
+        file_contents[schema_metadata.output_file] = python_code
 
     file_contents.update({Path(*module_path): result.body for module_path, result in parse_result.items()})
 
+    # add SQLModel classes for many-to-many relationships in "many.py"
+    if output_type is OutputType.SQL_MODEL.name:
+        shutil.rmtree(input_directory)  # remove intermediate dir of schemas
+        if links:
+            file_contents[Path("many.py")] = write_many_many_links(links)
+
     return file_contents
